﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DataAccess.Core.Data;

namespace DataAccess.Core.Attributes
{
    /// <summary>
    /// Indicates that one or more conventions about a data field need to be changed
    /// </summary>
    [AttributeUsage(AttributeTargets.Property)]
    public class DataFieldAttribute : Attribute
    {
        /// <summary>
        /// Defaults to property name
        /// </summary>
        public string FieldName { get; set; }

        /// <summary>
        /// Indicates if the field should be set on insert or not, defaults to true
        /// </summary>
        public bool SetOnInsert { get; set; }

        /// <summary>
        /// Indicates
        /// </summary>
        public bool LoadField { get; set; }

        /// <summary>
        /// Indicates the default value of this field, defaults to null
        /// </summary>
        public string DefaultValue { get; set; }

        /// <summary>
        /// Indicates the Data type of the data store field, set this if you want to override the default generation
        /// </summary>
        public string FieldType { get; set; }

        /// <summary>
        /// Indicates the type that this field relates to (FK relationship), defaults to null
        /// </summary>
        public Type PrimaryKeyType { get; set; }

        /// <summary>
        /// Indicates the type of fk relationship to generate
        /// </summary>
        public ForeignKeyType RelationshipType { get; set; }

        /// <summary>
        /// Default constructor
        /// </summary>
        public DataFieldAttribute()
        {
            SetOnInsert = true;
            LoadField = true;
            RelationshipType = ForeignKeyType.Cascade;
        }
    }
}
