﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DataAccess.DatabaseTests.Tests;
using DataAccess.Core.Interfaces;
using DataAccess.SQLite;
using Xunit;
using DataAccess.Core.Data;
using System.Data.SQLite;
using DataAccess.DatabaseTests.DataObjects;
using System.IO;

namespace DataAccess.DatabaseTests
{
    public class SqlLiteFunctionTests : FunctionTests
    {
        public override IDataStore GetDataStore()
        {
            return SqlLiteDataConnection.GetDataStore(Path.GetTempFileName());
        }

        public SqlLiteFunctionTests()
        {
            dstore = SqlLiteDataConnection.GetDataStore(Path.GetTempFileName());
            InitHelper.AttachEvents(dstore);

            List<DBObject> data = dstore.SchemaValidator.ViewValidator.GetObjects().Where(R => R.Name != "sqlite_sequence").ToList();
            while (data.Count != 0)
            {
                foreach (DBObject t in data)
                {
                    if (t.Name.Equals("sqlite_sequence")) continue;
                    SQLiteCommand cmd = new SQLiteCommand();
                    cmd.CommandText = string.Format("DROP VIEW {0}", t.Name);
                    try { dstore.ExecuteCommand(cmd); }
                    catch { }
                }
                data = dstore.SchemaValidator.ViewValidator.GetObjects(true).Where(R => R.Name != "sqlite_sequence").ToList();
            }

            data = dstore.SchemaValidator.TableValidator.GetObjects().Where(R => R.Name != "sqlite_sequence").ToList();
            while (data.Count != 0)
            {
                foreach (DBObject t in data)
                {
                    if (t.Name.Equals("sqlite_sequence")) continue;
                    SQLiteCommand cmd = new SQLiteCommand();
                    cmd.CommandText = string.Format("DROP TABLE {0}", t.Name);
                    try { dstore.ExecuteCommand(cmd); }
                    catch { }
                }
                data = dstore.SchemaValidator.TableValidator.GetObjects(true).Where(R => R.Name != "sqlite_sequence").ToList();
            }
        }

        public override void Test_Can_Get_Escape_Sequences()
        {
            Assert.True(dstore.Connection.LeftEscapeCharacter.Equals("\""));
            Assert.True(dstore.Connection.RightEscapeCharacter.Equals("\""));
        }

        public override void Test_Bulk_Insert()
        {
            List<TestItem> items = new List<TestItem>();
            for (int i = 0; i < 100; i++)
            {
                items.Add(new TestItem()
                {
                    id = i,
                    Something = "something",
                    TimeSpent = new TimeSpan(100)
                });
            }

            dstore.InsertObjects(items);

            List<TestItem> loaded = dstore.LoadEntireTable<TestItem>().ToList();
            Assert.True(loaded.Count == 100);
        }
    }
}
